@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -107,6 +110,42 @@ class EnvironmentsAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return EnvironmentsClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> EnvironmentsTransport:
         """Returns the transport used by the client instance.
@@ -179,6 +218,28 @@ class EnvironmentsAsyncClient:
     ) -> operation_async.AsyncOperation:
         r"""Create a new environment.
 
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_create_environment():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.CreateEnvironmentRequest(
+                )
+
+                # Make the request
+                operation = client.create_environment(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.CreateEnvironmentRequest, dict]):
                 The request object. Create a new environment.
@@ -210,7 +271,7 @@ class EnvironmentsAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, environment])
         if request is not None and has_flattened_params:
@@ -267,6 +328,24 @@ class EnvironmentsAsyncClient:
     ) -> environments.Environment:
         r"""Get an existing environment.
 
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_get_environment():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.GetEnvironmentRequest(
+                )
+
+                # Make the request
+                response = client.get_environment(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.GetEnvironmentRequest, dict]):
                 The request object. Get an environment.
@@ -291,7 +370,7 @@ class EnvironmentsAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -338,6 +417,25 @@ class EnvironmentsAsyncClient:
     ) -> pagers.ListEnvironmentsAsyncPager:
         r"""List environments.
 
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_list_environments():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.ListEnvironmentsRequest(
+                )
+
+                # Make the request
+                page_result = client.list_environments(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.ListEnvironmentsRequest, dict]):
                 The request object. List environments in a project and
@@ -366,7 +464,7 @@ class EnvironmentsAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -421,6 +519,28 @@ class EnvironmentsAsyncClient:
     ) -> operation_async.AsyncOperation:
         r"""Update an environment.
 
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_update_environment():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.UpdateEnvironmentRequest(
+                )
+
+                # Make the request
+                operation = client.update_environment(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.UpdateEnvironmentRequest, dict]):
                 The request object. Update an environment.
@@ -643,7 +763,7 @@ class EnvironmentsAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, environment, update_mask])
         if request is not None and has_flattened_params:
@@ -702,6 +822,28 @@ class EnvironmentsAsyncClient:
     ) -> operation_async.AsyncOperation:
         r"""Delete an environment.
 
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_delete_environment():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.DeleteEnvironmentRequest(
+                )
+
+                # Make the request
+                operation = client.delete_environment(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.DeleteEnvironmentRequest, dict]):
                 The request object. Delete an environment.
@@ -739,7 +881,7 @@ class EnvironmentsAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -793,6 +935,28 @@ class EnvironmentsAsyncClient:
     ) -> operation_async.AsyncOperation:
         r"""Restart Airflow web server.
 
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_restart_web_server():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.RestartWebServerRequest(
+                )
+
+                # Make the request
+                operation = client.restart_web_server(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.RestartWebServerRequest, dict]):
                 The request object. Restart Airflow web server.
@@ -855,6 +1019,29 @@ class EnvironmentsAsyncClient:
         In case of problems detailed info can be found in the
         returned Operation.
 
+
+        .. code-block:: python
+
+            from google.cloud.orchestration.airflow import service_v1beta1
+
+            def sample_check_upgrade():
+                # Create a client
+                client = service_v1beta1.EnvironmentsClient()
+
+                # Initialize request argument(s)
+                request = service_v1beta1.CheckUpgradeRequest(
+                )
+
+                # Make the request
+                operation = client.check_upgrade(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.orchestration.airflow.service_v1beta1.types.CheckUpgradeRequest, dict]):
                 The request object. Request to check whether image
